﻿-- UI FUNCTIONS --
function PoJ_CreateActionButtonLabel(button)
  local string
  if button then
    buttonname = button:GetName()
    stringname = buttonname .. "PojText"
    if not _G[stringname] then
      string = button:CreateFontString(stringname, "OVERLAY", "NumberFontNormal")
      string:SetPoint(_G[buttonname .. "Count"]:GetPoint(1))
      string:SetJustifyH("RIGHT")
      table.insert(PoJ.WoWActionButtons, button)
    end
  end
end


function PoJ_Drag_Start(object, condition)
  if condition then
    object.drag = true
    object:StartMoving()
  end
end


function PoJ_Drag_Stop(object)
  object.drag = nil
  object:StopMovingOrSizing()
end


function PoJ_EnableButton(button, enable)
  if enable then
    button:Enable()
  else
    button:Disable()
  end
end


function PoJ_GameTooltipSetUnit(self, unit, buff, filter)
  if PoJ_Vars.ShowBuffer then
    local _, _, _, _, _, _, source, _, _, _, _, _, castByPlayer = UnitAura(unit, buff, filter)
    if source then
      local name = UnitName(source)
      if name then
        local color
        if castByPlayer then
          local _, class = UnitClass(source)
          if class then
            color = RAID_CLASS_COLORS[class]
          end
        end
		if color == nil then
		  color = {}
		  color.r, color.g, color.b = PoJ_GetFactionColor(source, 0.25)
		end
        self:AddLine(name, color.r, color.g, color.b)
        GameTooltip:Show()
	  end
    end
  end
end


function PoJ_GameTooltipSetUnitAura(self, unit, buff, filter)
  PoJ_GameTooltipSetUnit(self, unit, buff, filter)
end


function PoJ_GameTooltipSetUnitBuff(self, unit, buff)
  PoJ_GameTooltipSetUnit(self, unit, buff, "HELPFUL")
end


function PoJ_GameTooltipSetUnitDebuff(self, unit, buff)
  PoJ_GameTooltipSetUnit(self, unit, buff, "HARMFUL")
end


function PoJ_GetWoWActionSpellName(action)
  local _, spellname
  local spelltype, spellid = GetActionInfo(action)
  if spelltype == BOOKTYPE_SPELL then
    spellname = GetSpellInfo(spellid)
  elseif spelltype == "macro" and spellid then
    local text = GetMacroBody(spellid)
    if text then
      text = strmatch(text, "#showtooltip +([^%c]+)%c")
      if text then
        spellname = GetSpellInfo(text)
      end
	else
	  spellname = "~macro~"
    end
  end
  return spellname
end


function PoJ_MinimapIcon_SetTooltip()
  PoJ_ShowTooltip(PoJ_Minimap_Icon, "Piece of Justice")
end


function PoJ_MinimapIcon_Update(self)
  if self.drag then
    local top = Minimap:GetTop()
    local right = Minimap:GetRight()
    local width = Minimap:GetWidth()
    local height = Minimap:GetHeight()
    local centerX = right - width / 2
    local centerY = top - height / 2
    local minDistance = (width + height) / 4 + 6
    local maxDistance = minDistance + 25
    local scale = UIParent:GetScale()
    local x, y = GetCursorPosition()
    x = x / scale - centerX
    y = y / scale - centerY
    local distance = sqrt(x^2 + y^2)
    if distance <= minDistance then
      local factor = minDistance / distance
      x = x * factor
      y = y * factor
    elseif distance >= maxDistance then
      local factor = maxDistance / distance
      x = x * factor
      y = y * factor
    end
    PoJ_SetMinimapIconPos(x, y)
  end
end


function PoJ_SetFontColor(fontstring, enable)
  if enable then
    fontstring:SetTextColor(1, 0.8, 0, 1)
  else
    fontstring:SetTextColor(0.5, 0.5, 0.5, 1)
  end
end


function PoJ_SetMinimapIconPos(x, y)
  PoJ_Vars.MinimapX = x
  PoJ_Vars.MinimapY = y
  PoJ_Minimap_Icon:SetPoint("CENTER", Minimap, "CENTER", x, y - 2)
end


function PoJ_ShowBottomtip(tooltip, head, text, errortext)
  local parent = UIParent
  local show = false
  tooltip:ClearLines()
  tooltip:ClearAllPoints()
  if not parent or not parent:IsVisible() then
    parent = UIParent
  end
  GameTooltip_SetDefaultAnchor(tooltip, parent)
  if head and head ~= "" then
    tooltip:AddLine(head, 1, 1, 1, 1)
    show = true
  end
  if errortext and errortext ~= "" then
    tooltip:AddLine(errortext, 1, 0, 0, 1)
    show = true
  end
  if text and text ~= "" then
    tooltip:AddLine(text, 1, 0.82, 0, 1)
    show = true
  end
  if show then
    tooltip:Show()
  end
end


function PoJ_ShowMinimapIcon()
  PoJ_ShowObject(PoJ_Minimap_Icon, PoJ_Vars.MinimapIcon)
end


function PoJ_ShowObject(object, show)
  if show then
    object:Show()
  else
    object:Hide()
  end
end


function PoJ_ShowTooltip(self, head, text)
  local objectname = self:GetName()
  GameTooltip:SetOwner(_G[objectname], "ANCHOR_TOP")
  GameTooltip:ClearLines()
  GameTooltip:ClearAllPoints()
  GameTooltip:SetPoint("TOPRIGHT", objectname, "BOTTOMRIGHT", -2, 2)
  if head and head ~= "" then
    GameTooltip:AddLine(head)
  end
  if text and text ~= "" then
    GameTooltip:AddLine(text, 1, 1, 1, 1)
  end
  GameTooltip:Show()
end


function PoJ_TargetIconButton_OnClick(self, button)
  local index = self:GetID()
  self:SetChecked(false)
  if GetRaidTargetIndex("target") == index then
    index = 0
  end
  SetRaidTarget("target", index)
end


function PoJ_TargetIconButton_OnLoad(self)
  self:BaseActionButtonMixin_OnLoad()
  local width, height = self:GetSize()
  local icon = _G[self:GetName() .. "Icon"]
  icon:SetSize(width, height)
  icon:ClearAllPoints()
  icon:SetPoint("TOPLEFT", self, "TOPLEFT", 1.5, -1.5)
  self.NormalTexture:SetSize(width + 5, height + 5)
  self.HighlightTexture:SetSize(width + 4, height + 4)
  self.PushedTexture:SetSize(width + 5, height + 5)
  icon:SetTexture("Interface\\TargetingFrame\\UI-RaidTargetingIcons")
  SetRaidTargetIconTexture(icon, self:GetID())
  self:RegisterForClicks("LeftButtonUp", "RightButtonUp")
end


function PoJ_ToggleFrame(frame, uiframe)
  if frame:IsVisible() then
    if uiframe and not InCombatLockdown() then
      HideUIPanel(frame)
    else
      frame:Hide()
    end
  elseif not InCombatLockdown() then
    if uiframe then
      ShowUIPanel(frame)
    else
      frame:Show()
    end
  end
end


function PoJ_UpdateAllActionButtons()
  for _, button in ipairs(PoJ.WoWActionButtons) do
    button:Update()
  end
end


function PoJ_UpdateActionButtonMacroName(button)
  local macroname = _G[button:GetName().."Name"]
  if macroname then
    if not PoJ_Vars.NoMacroNames and not IsConsumableAction(button.action) and not IsStackableAction(button.action) then
      macroname:SetText(GetActionText(button.action))
    else
      macroname:SetText("")
    end
  end
  PoJ_CreateActionButtonLabel(button)
end


function PoJ_UpdateTargetClass()
  PoJ_TargetClassName:Hide()
  if PoJ_Vars.PvPClasses and UnitExists("target") and UnitIsPlayer("target") and UnitIsEnemy("target", "player") then
    local locclass = UnitClass("target")
    for class, classname in pairs(POJ_STRING.CLASS) do
      if classname == locclass then
        local color = RAID_CLASS_COLORS[class]
        PoJ_TargetClassNameText:SetText(locclass)
        PoJ_TargetClassNameText:SetTextColor(color.r, color.g, color.b)
        PoJ_TargetClassName:Show()
      end
    end
  end
end


function PoJ_UpdateTargetIcons()
  PoJ_ShowObject(PoJ_TargetIcons, PoJ_Vars.RaidIcons and UnitExists("target") and (UnitIsFriend("target", "player") or not UnitIsPlayer("target")) --[[and (PoJ_GetGroupType() == "party" or IsRaidLeader() or IsRaidOfficer())]])
end


function PoJ_UpdateWoWActionButtons()
  if PoJ_Vars.ButtonCooldownsShown or PoJ_Vars.ShowActionCD or PoJ_Vars.ShowActionOT or PoJ_Vars.ShowActionBD then
    local shown
    for _, button in ipairs(PoJ.WoWActionButtons) do
      shown = PoJ_WoWActionButtonUpdate(button) or shown
    end
    PoJ_Vars.ButtonCooldownsShown = shown
  end
end


function PoJ_UpdateWoWActionSpellName(action)
  PoJ.WoWActionSpellName[action] = PoJ_GetWoWActionSpellName(action)
end


function PoJ_UpdateWoWActionSpellNames()
  local action
  for action = 1, 132 do
    table.insert(PoJ.WoWActionSpellName, action, PoJ_GetWoWActionSpellName(action))
  end
end


function PoJ_WoWActionButtonUpdate(button)
  if button and button:IsVisible() then
    local hasCount = IsConsumableAction(button.action) or IsStackableAction(button.action)
    local maxtime = 0
    local maxtype
    local showtime, timetype
    if not hasCount and (PoJ_Vars.ShowActionCD or PoJ_Vars.ShowActionOT or PoJ_Vars.ShowActionBD) then
      local action = button.action
      if PoJ_Vars.ShowActionCD then
        local start, duration, enable = GetActionCooldown(action)
        if start > 0 and duration > 2 then
          showtime = PoJ_GetCooldownEndTime(start, duration) - GetTime()
          if showtime > 0 then
            maxtime = showtime
            maxtype = "cooldown"
          end
        end
      end
      if PoJ_Vars.ShowActionOT or PoJ_Vars.ShowActionBD then
        local _, casttime, spellname
        local spelltype, spellid = GetActionInfo(action)
        showtime = nil
        if spelltype == BOOKTYPE_SPELL then
          spellname, _, _, casttime = GetSpellInfo(spellid)
        elseif spelltype == "macro" and spellid then
          spellname = PoJ.WoWActionSpellName[action]
          if spellname then
            if spellname == "~macro~" then
              PoJ_UpdateWoWActionSpellName(action)
              spellname = PoJ.WoWActionSpellName[action]
            end
            spellname, _, _, casttime = GetSpellInfo(spellname)
          end
        end
        if spellname then
          if PoJ_Vars.ShowActionOT and UnitCanAttack("player", "target") then
            local debuff = PoJ_GetDebuffName(spellname)
            showtime = PoJ_GetAuraTimeLeft("target", debuff, "HARMFUL", true)
            if showtime then
              timetype = "debuff"
              if showtime == -1 then
                if not maxtype then
                  maxtime = showtime
                  maxtype = timetype
                end
              else
                if casttime < 0 then
                  casttime = 0
                end
                showtime = showtime - casttime / 1000
                if showtime > 0 and (not maxtype or showtime > maxtime) then
                  maxtime = showtime
                  maxtype = timetype
                end
              end
            end
          end
          if PoJ_Vars.ShowActionBD and not (IsConsumableAction(action) or IsStackableAction(action)) then
            local unit = "player"
            if ActionHasRange(action) and UnitCanAssist("player", "target") then
              unit = "target"
            end
            local buff = PoJ_GetBuffName(spellname)
            showtime = PoJ_GetAuraTimeLeft(unit, buff, "HELPFUL", true)
            if showtime and showtime > 0 then
              maxtime = showtime
              maxtype = "buff"
            end
          end
        end
      end
    end
    local showmacrodot = not maxtype and PoJ_Vars.NoMacroNames and not hasCount and GetActionText(button.action)
    local string = _G[button:GetName() .. "PojText"]
    if string then
      if maxtype then
        local color = PoJ.ActionbarTimeColors[maxtype]
        string:SetTextColor(color.r, color.g, color.b)
        string:SetText(PoJ_GetTimeString(maxtime, true))
        string:Show()
        return true
      elseif showmacrodot then
        string:SetTextColor(1, 1, 1)
        string:SetText(".")
        string:Show()
      else
        string:Hide()
      end
    end
  end
end
